@@ -270,22 +270,6 @@ get_item_widget (GtkWidget *widget)
 	return NULL;
 }
 
-static GdkPixbuf *
-cmd_get_menu_pixbuf (GtkWidget    *widget,
-		     BonoboUINode *node,
-		     BonoboUINode *cmd_node)
-{
-	if (bonobo_ui_node_peek_attr (node, "pixtype"))
-		return bonobo_ui_util_xml_get_pixbuf (
-			widget, node, GTK_ICON_SIZE_MENU);
-
-	if (bonobo_ui_node_peek_attr (cmd_node, "pixtype"))
-		return bonobo_ui_util_xml_get_pixbuf (
-			widget, cmd_node, GTK_ICON_SIZE_MENU);
-
-	return NULL;
-}
-
 static gboolean
 label_same (GtkBin *menu_widget, const char *txt)
 {
@@ -336,32 +320,30 @@ impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 	if ((type = bonobo_ui_engine_get_attr (node, cmd_node, "type")))
 		bonobo_ui_node_free_string (type);
 	else {
-		GdkPixbuf        *pixbuf;
 		GtkImageMenuItem *image_menu_item;
 		
 		image_menu_item = (GtkImageMenuItem *) menu_widget;
 		
-		if (!bonobo_ui_preferences_get_menus_have_icons ()) {
+		if (!bonobo_ui_preferences_get_menus_have_icons ())
 			gtk_image_menu_item_set_image (image_menu_item, NULL);
-		}
-		else {
-			
-			pixbuf = cmd_get_menu_pixbuf (menu_widget, node, cmd_node);
-			
-			if (pixbuf) {
-				GtkWidget *image;
-				
-				image = gtk_image_menu_item_get_image (image_menu_item);
-				
-				if (!image) {
-					image = gtk_image_new_from_pixbuf (pixbuf);
-					gtk_widget_show (image);
-					g_object_set (G_OBJECT (menu_widget), "image",
-						      image, NULL);
-				} else
-					bonobo_ui_image_set_pixbuf (
-						GTK_IMAGE (image), pixbuf);
+
+		else if (bonobo_ui_node_peek_attr (node, "pixtype") ||
+			 bonobo_ui_node_peek_attr (cmd_node, "pixtype")) {
+			GtkWidget *image;
+
+			image = gtk_image_menu_item_get_image (image_menu_item);
+		
+			if (!image) {
+				image = gtk_image_new ();
+				g_object_set (G_OBJECT (image_menu_item),
+					      "image", image, NULL);
 			}
+
+			bonobo_ui_util_xml_set_image (
+				GTK_IMAGE (image), node, cmd_node,
+				GTK_ICON_SIZE_MENU);
+
+			gtk_widget_show (image);
 		}
 	}
 	
@@ -635,13 +617,13 @@ impl_bonobo_ui_sync_menu_build (BonoboUISync     *sync,
 		
 		shell = GTK_MENU_SHELL (parent);
 		g_signal_connect (GTK_OBJECT (shell), "key_press_event",
-				    (GtkSignalFunc) sucking_gtk_keybindings_cb, NULL);
+				  (GtkSignalFunc) sucking_gtk_keybindings_cb, NULL);
 
 
 		/* Create the menu shell. */
 		menu = GTK_MENU (gtk_menu_new ());
 		g_signal_connect (GTK_OBJECT (menu), "key_press_event",
-				    (GtkSignalFunc) sucking_gtk_keybindings_cb, NULL);
+				  (GtkSignalFunc) sucking_gtk_keybindings_cb, NULL);
 
 		gtk_menu_set_accel_group (menu, menu_sync->accel_group);
 
@@ -668,7 +650,7 @@ impl_bonobo_ui_sync_menu_build (BonoboUISync     *sync,
 				    (GtkSignalFunc) exec_verb_cb, engine);
 
 	g_signal_connect (GTK_OBJECT (menu_widget), "key_press_event",
-			    (GtkSignalFunc) sucking_gtk_keybindings_cb, NULL);
+			  (GtkSignalFunc) sucking_gtk_keybindings_cb, NULL);
 
 	gtk_widget_show (menu_widget);
 			    
@@ -740,6 +722,7 @@ impl_dispose (GObject *object)
 	BonoboUISyncMenu *sync = (BonoboUISyncMenu *) object;
 
 	if (sync->menu) {
+		gtk_widget_destroy (GTK_WIDGET (sync->menu));
 		g_object_unref (sync->menu);
 		sync->menu = NULL;
 	}
@@ -850,6 +833,21 @@ impl_bonobo_ui_sync_menu_get_attached (BonoboUISync *sync,
 	return get_item_widget (widget);
 }
 
+GtkWidget *
+impl_bonobo_ui_sync_menu_wrap_widget (BonoboUISync *sync,
+				      GtkWidget    *custom_widget)
+{
+	if (!GTK_IS_MENU_ITEM (custom_widget)) {
+		GtkWidget *widget;
+
+		widget = gtk_menu_item_new ();
+		gtk_container_add (GTK_CONTAINER (widget),
+				   custom_widget);
+		return widget;
+	} else 
+		return custom_widget;
+}
+
 static void
 class_init (BonoboUISyncClass *sync_class)
 {
@@ -873,6 +871,7 @@ class_init (BonoboUISyncClass *sync_class)
 	sync_class->can_handle    = impl_bonobo_ui_sync_menu_can_handle;
 
 	sync_class->get_attached  = impl_bonobo_ui_sync_menu_get_attached;
+	sync_class->wrap_widget   = impl_bonobo_ui_sync_menu_wrap_widget;
 }
 
 static void
@@ -927,7 +926,7 @@ bonobo_ui_sync_menu_new (BonoboUIEngine *engine,
 		g_object_ref (menu_dock_item) :
 		menu_dock_item;
 
-	sync->accel_group = group ? g_object_ref (group) : NULL;
+	sync->accel_group = group ? g_object_ref (group) : gtk_accel_group_new ();
 
 	return bonobo_ui_sync_construct (
 		BONOBO_UI_SYNC (sync), engine, TRUE, TRUE);
